# coding: utf-8
from __future__ import unicode_literals

import os
import re

from sorl.thumbnail import get_thumbnail
from django.conf import settings


URLS_RE = re.compile(
    r'src="(.*?)" style=".*?height:(\d+)px;.*?width:(\d+)px',
    re.I | re.U | re.M)


def create_thumbs(value):
    """
    Parse given html string for search img tag
    with height and width styles
    """
    chunks = URLS_RE.findall(value)

    if not chunks:
        return value

    for chunk in chunks:
        if len(chunk) != 3:
            continue

        url, height, width = chunk

        img_path = os.path.join(settings.BASE_DIR, url[url.startswith('/'):])
        thumb = get_thumbnail(img_path, '{}x{}'.format(width, height))

        value = value.replace(url, thumb.url)

    return value



# coding: utf-8
from __future__ import unicode_literals

from django.db import models
from django.conf import settings
from django.utils.safestring import mark_safe

from sorl.thumbnail import get_thumbnail


__ALL__ = ('ThumbMixin',)


THUMBNAIL_OPTIONS = getattr(
    settings,
    'THUMBNAIL_OPTIONS',
    dict(crop='center', upscaled=False, quality=100))

THUMBNAIL_EMPTY = getattr(settings, 'THUMBNAIL_EMPTY', None)

THUMBNAIL_ALIASES = getattr(settings, 'THUMBNAIL_ALIASES', dict(small='20x20'))


class ThumbMixin(object):
    """
    Add property to model for generated thumb.
    Usefull in Django templates:

    ::

        {{ my_model.get_thumb_56x56.html }}

        {{ my_model.get_thumb_small.html }}

    instead of:

    ::
        {% thumbnail my_model.pic "56x56" crop="center" as im %}
            <img src="{{ im.url }}" width="56" height="56">
        {% empty %}
            <img src="{{ MISSING_IMAGE }}" width="56" height="56">
        {% endthumbnail %}
    """
    @property
    def image_file_field(self):
        try:
            return self.image
        except AttributeError:
            raise NotImplementedError('Set up image_file_field property')

    def __getattr__(self, name):
        prefix = 'get_thumb_'
        if name.startswith(prefix):
            suffix = name.replace(prefix, '')

            if THUMBNAIL_ALIASES and suffix in THUMBNAIL_ALIASES:
                size = THUMBNAIL_ALIASES[suffix]
            else:
                size = suffix

            return self._get_thumb(size)
        raise AttributeError(name)

    def _get_thumb(self, size):
        if not self.image_file_field:
            return THUMBNAIL_EMPTY

        tmpl = '<img src="{0.url}" width="{0.width}" height="{0.height}">'
        thumb = get_thumbnail(
            self.image_file_field.file, size, **THUMBNAIL_OPTIONS)
        thumb.html = mark_safe(tmpl.format(thumb))
        return thumb



# coding: utf-8
from __future__ import unicode_literals

from django.contrib import admin
from django.db.models import TextField

from .utils import create_thumbs


class ThumbedCkeditorImages(admin.ModelAdmin):
    class Meta:
        abstract = True

    def save_model(self, request, obj, form, change):
        for fname in form.changed_data:
            field = obj._meta.get_field_by_name(fname)[0]
            if isinstance(field, TextField):
                setattr(obj, fname, create_thumbs(getattr(obj, fname)))

        super(ThumbedCkeditorImages, self).save_model(
            request, obj, form, change)



# coding: utf-8
from __future__ import unicode_literals

from django import template
from django.template.defaultfilters import stringfilter

from ..utils import create_thumbs as _create_thumbs


register = template.Library()


@register.filter
@stringfilter
def create_thumbs(value):
    """
    Replace images by thumbs `on fly`
    """
    return _create_thumbs(value)



# coding: utf-8
from __future__ import unicode_literals

from django.core.management.base import AppCommand, CommandError
from django.db.models import get_models
from django.db.models import TextField

from sorl_hacks.utils import create_thumbs


class Command(AppCommand):
    help = 'Generate thumbs for models of app'

    def handle_app(self, app, **kwargs):
        for model in get_models(app):
            affected_fields = [f.name for f in model._meta.fields
                               if isinstance(f, TextField)]

            if not affected_fields:
                continue

            self.proccess(model, affected_fields)

    def proccess(self, model, fields):
        i = 0
        for i, obj in enumerate(model.objects.all()):
            for fname in fields:
                setattr(obj, fname, create_thumbs(getattr(obj, fname)))
            obj.save()

        print('>> {} {} updated'.format(i, repr(model)))



# coding: utf-8
from setuptools import setup, find_packages

setup(
    name='django-sorl-hacks',
    version='0.1',
    author='Vladimir Savin',
    author_email='zero13cool@yandex.ru',

    description='Some usefull hacks based on sorl-thumbnail',
    long_description=open('README.rst').read(),
    url='https://github.com/zerc/sorl_hacks',
    license='MIT',

    packages=find_packages(exclude=['ez_setup', 'tests', 'tests.*']),
    include_package_data=True,
    install_requires=[
        'django>=1.6',
        'sorl-thumbnail==11.12',
    ],

    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',

        'Framework :: Django',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)
